@@ -1,4 +1,4 @@
-// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef BTCONNECTIONMONITOR_P_H
@@ -15,9 +15,10 @@
 // We mean it.
 //
 
-#include "qbluetoothaddress.h"
 #include "btdelegates_p.h"
 
+#include <QtCore/private/qcore_mac_p.h>
+
 #include <QtCore/qglobal.h>
 
 #include <Foundation/Foundation.h>
@@ -33,4 +34,6 @@
 - (void)stopMonitoring;
 @end
 
+QT_NAMESPACE_ALIAS_OBJC_CLASS(DarwinBTConnectionMonitor);
+
 #endif
