@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtBluetooth module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qbluetoothserviceinfo.h"
 #include "qbluetoothserviceinfo_p.h"
@@ -44,6 +8,11 @@
 
 QT_BEGIN_NAMESPACE
 
+QT_IMPL_METATYPE_EXTERN(QBluetoothServiceInfo)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QBluetoothServiceInfo::Sequence, QBluetoothServiceInfo__Sequence)
+QT_IMPL_METATYPE_EXTERN_TAGGED(QBluetoothServiceInfo::Alternative,
+                               QBluetoothServiceInfo__Alternative)
+
 /*!
     \class QBluetoothServiceInfo::Sequence
     \inmodule QtBluetooth
@@ -494,7 +463,7 @@ int QBluetoothServiceInfo::protocolServiceMultiplexer() const
 
     if (parameters.isEmpty())
         return -1;
-    else if (parameters.count() == 1)
+    else if (parameters.size() == 1)
         return 0;
     else
         return parameters.at(1).toUInt();
@@ -541,7 +510,7 @@ QList<QBluetoothUuid> QBluetoothServiceInfo::serviceClassUuids() const
         return results;
 
     const QBluetoothServiceInfo::Sequence seq = var.value<QBluetoothServiceInfo::Sequence>();
-    for (int i = 0; i < seq.count(); i++)
+    for (qsizetype i = 0; i < seq.size(); ++i)
         results.append(seq.at(i).value<QBluetoothUuid>());
 
     return results;
@@ -558,6 +527,7 @@ QBluetoothServiceInfo &QBluetoothServiceInfo::operator=(const QBluetoothServiceI
     return *this;
 }
 
+#ifndef QT_NO_DEBUG_STREAM
 static void dumpAttributeVariant(QDebug dbg, const QVariant &var, const QString& indent)
 {
     switch (var.typeId()) {
@@ -615,7 +585,7 @@ static void dumpAttributeVariant(QDebug dbg, const QVariant &var, const QString&
             case 16:
                 dbg << QString::asprintf("%suuid %s\n",
                             indent.toUtf8().constData(),
-                            QByteArray(reinterpret_cast<const char *>(uuid.toUInt128().data), 16).toHex().constData());
+                            uuid.toByteArray(QUuid::Id128).constData());
                 break;
             default:
                 dbg << QString::asprintf("%suuid ???\n", indent.toUtf8().constData());
@@ -636,7 +606,6 @@ static void dumpAttributeVariant(QDebug dbg, const QVariant &var, const QString&
     }
 }
 
-#ifndef QT_NO_DEBUG_STREAM
 QDebug QBluetoothServiceInfo::streamingOperator(QDebug dbg, const QBluetoothServiceInfo &info)
 {
     QDebugStateSaver saver(dbg);
@@ -647,7 +616,7 @@ QDebug QBluetoothServiceInfo::streamingOperator(QDebug dbg, const QBluetoothServ
     }
     return dbg;
 }
-#endif
+#endif // QT_NO_DEBUG_STREAM
 
 QBluetoothServiceInfo::Sequence QBluetoothServiceInfoPrivate::protocolDescriptor(QBluetoothUuid::ProtocolUuid protocol) const
 {
@@ -675,7 +644,7 @@ int QBluetoothServiceInfoPrivate::serverChannel() const
 
     if (parameters.isEmpty())
         return -1;
-    else if (parameters.count() == 1)
+    else if (parameters.size() == 1)
         return 0;
     else
         return parameters.at(1).toUInt();
