@@ -14,6 +14,7 @@
 #include <QtCore/QThread>
 #include <QtCore/QTime>
 #include <QtCore/QJniEnvironment>
+#include <QtCore/q26numeric.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -607,14 +608,15 @@ qint64 QBluetoothSocketPrivateAndroid::writeData(const char *data, qint64 maxSiz
         return -1;
     }
 
+    const qint32 javaSize = q26::saturate_cast<qint32>(maxSize);
     QJniEnvironment env;
-    jbyteArray nativeData = env->NewByteArray((qint32)maxSize);
-    env->SetByteArrayRegion(nativeData, 0, (qint32)maxSize, reinterpret_cast<const jbyte*>(data));
+    jbyteArray nativeData = env->NewByteArray(javaSize);
+    env->SetByteArrayRegion(nativeData, 0, javaSize, reinterpret_cast<const jbyte*>(data));
     auto methodId = env.findMethod(outputStream.objectClass(),
                                    "write",
                                    "([BII)V");
     if (methodId)
-        env->CallVoidMethod(outputStream.object(), methodId, nativeData, 0, (qint32)maxSize);
+        env->CallVoidMethod(outputStream.object(), methodId, nativeData, 0, javaSize);
     env->DeleteLocalRef(nativeData);
 
     if (!methodId || env.checkAndClearExceptions()) {
@@ -624,8 +626,8 @@ qint64 QBluetoothSocketPrivateAndroid::writeData(const char *data, qint64 maxSiz
         return -1;
     }
 
-    emit q->bytesWritten(maxSize);
-    return maxSize;
+    emit q->bytesWritten(javaSize);
+    return javaSize;
 }
 
 qint64 QBluetoothSocketPrivateAndroid::readData(char *data, qint64 maxSize)
