@@ -132,6 +132,8 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
     \value [since 6.4] MissingPermissionsError  The operating system requests
                                                 permissions which were not
                                                 granted by the user.
+    \value [since 6.5] RssiReadError An attempt to read RSSI (received signal strength indicator)
+                                     of a remote device finished with error.
 */
 
 /*!
@@ -195,7 +197,8 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
     \l PeripheralRole. On iOS and macOS the controller only guesses that some central
     connected to our peripheral as soon as this central tries to write/read a
     characteristic/descriptor. On Android the controller monitors all connected GATT
-    devices.
+    devices. On Linux BlueZ DBus peripheral backend the remote is considered connected
+    when it first reads/writes a characteristic or a descriptor.
 */
 
 /*!
@@ -211,13 +214,26 @@ Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
     \sa mtu()
 */
 
+/*!
+    \fn void QLowEnergyController::rssiRead(qint16 rssi)
+
+    This signal is emitted after successful read of RSSI (received
+    signal strength indicator) for a connected remote device.
+    \a rssi parameter represents the new value.
+
+    \sa readRssi()
+    \since 6.5
+*/
+
 /*!
     \fn void QLowEnergyController::disconnected()
 
     This signal is emitted when the controller disconnects from the remote
     Low Energy device or vice versa. On iOS and macOS this signal is unreliable
     if the controller is in the \l PeripheralRole. On Android the signal is emitted
-    when the last connected device is disconnected.
+    when the last connected device is disconnected. On BlueZ DBus backend the controller
+    is considered disconnected when last client which has accessed the attributes has
+    disconnected.
 */
 
 /*!
@@ -288,30 +304,58 @@ void registerQLowEnergyControllerMetaType()
     }
 }
 
-static QLowEnergyControllerPrivate *privateController(QLowEnergyController::Role role)
+static QLowEnergyControllerPrivate *privateController(
+        QLowEnergyController::Role role,
+        const QBluetoothAddress& localDevice = QBluetoothAddress{})
 {
 #if QT_CONFIG(bluez) && !defined(QT_BLUEZ_NO_BTLE)
-    // The new DBUS implementation only supports Central role for now
-    // For Peripheral role support see QTBUG-66909
+    // Central role
+    // The minimum Bluez DBus version for central role is 5.42, otherwise we
+    // fall back to kernel ATT backend. Application can also force the choice
+    // with an environment variable (see bluetoothdVersion())
+    //
+    // Peripheral role
+    // The DBus peripheral backend is the default, and for that we check the presence of
+    // the required DBus APIs and bluez version. The application may opt-out of the DBus
+    // peripheral role by setting the environment variable. The fall-back is the kernel ATT
+    // backend
+    //
+    // ### Qt 7 consider removing the non-dbus bluez (kernel ATT) support
+
+    QString adapterPathWithPeripheralSupport;
+    if (role == QLowEnergyController::PeripheralRole
+            && bluetoothdVersion() >= QVersionNumber(5, 56)
+            && !qEnvironmentVariableIsSet("QT_BLUETOOTH_USE_KERNEL_PERIPHERAL")) {
+        adapterPathWithPeripheralSupport = adapterWithDBusPeripheralInterface(localDevice);
+    }
+
     if (role == QLowEnergyController::CentralRole
             && bluetoothdVersion() >= QVersionNumber(5, 42)) {
-        qCWarning(QT_BT) << "Using BlueZ LE DBus API";
+        qCDebug(QT_BT) << "Using BlueZ LE DBus API for central";
         return new QLowEnergyControllerPrivateBluezDBus();
+    } else if (!adapterPathWithPeripheralSupport.isEmpty()) {
+        qCDebug(QT_BT) << "Using BlueZ LE DBus API for peripheral";
+        return new QLowEnergyControllerPrivateBluezDBus(adapterPathWithPeripheralSupport);
     } else {
-        qCWarning(QT_BT) << "Using BlueZ kernel ATT interface";
+        qCDebug(QT_BT) << "Using BlueZ kernel ATT interface for"
+                       << (role == QLowEnergyController::CentralRole ? "central" : "peripheral");
         return new QLowEnergyControllerPrivateBluez();
     }
 #elif defined(QT_ANDROID_BLUETOOTH)
     Q_UNUSED(role);
+    Q_UNUSED(localDevice);
     return new QLowEnergyControllerPrivateAndroid();
 #elif defined(QT_WINRT_BLUETOOTH)
     Q_UNUSED(role);
+    Q_UNUSED(localDevice);
     return new QLowEnergyControllerPrivateWinRT();
 #elif defined(Q_OS_DARWIN)
     Q_UNUSED(role);
+    Q_UNUSED(localDevice);
     return new QLowEnergyControllerPrivateDarwin();
 #else
     Q_UNUSED(role);
+    Q_UNUSED(localDevice);
     return new QLowEnergyControllerPrivateCommon();
 #endif
 }
@@ -437,7 +481,7 @@ QLowEnergyController *QLowEnergyController::createPeripheral(const QBluetoothAdd
 QLowEnergyController::QLowEnergyController(const QBluetoothAddress &localDevice, QObject *parent)
     : QObject(parent)
 {
-    d_ptr = privateController(PeripheralRole);
+    d_ptr = privateController(PeripheralRole, localDevice);
 
     Q_D(QLowEnergyController);
     d->q_ptr = this;
@@ -720,6 +764,10 @@ QLowEnergyService *QLowEnergyController::createServiceObject(
    the advertised packets may not contain all uuids. The existing limit may have caused the truncation
    of uuids. In such cases \a scanResponseData may be used for additional information.
 
+   On BlueZ DBus backend BlueZ decides if, and which data, to use in a scan response. Therefore
+   all advertisement data is recommended to set in the main \a advertisingData parameter. If both
+   advertisement and scan response data is set, the scan response data is given precedence.
+
    If this object is currently not in the \l UnconnectedState, nothing happens.
 
    \since 5.7
@@ -792,7 +840,7 @@ QLowEnergyService *QLowEnergyController::addService(const QLowEnergyServiceData
     }
 
 #if defined(QT_ANDROID_BLUETOOTH)
-    if (!ensureAndroidPermission(BluetoothPermission::Connect)) {
+    if (!ensureAndroidPermission(QBluetoothPermission::Access)) {
         qCWarning(QT_BT_ANDROID) << "addService() failed due to missing permissions";
         return nullptr;
     }
@@ -823,7 +871,7 @@ QLowEnergyService *QLowEnergyController::addService(const QLowEnergyServiceData
   an acknowledged Android bug. Due to this bug Android does not emit the \l connectionUpdated()
   signal.
 
-  \note Currently, this functionality is only implemented on Linux and Android.
+  \note Currently, this functionality is only implemented on Linux kernel backend and Android.
 
   \sa connectionUpdated()
   \since 5.7
@@ -898,6 +946,38 @@ int QLowEnergyController::mtu() const
     return d_ptr->mtu();
 }
 
+/*!
+    readRssi() reads RSSI (received signal strength indicator) for a connected remote device.
+    If the read was successful, the RSSI is then reported by rssiRead() signal.
+
+    \note Prior to calling readRssi(), this controller must be connected to a peripheral.
+    This controller must be created using createCentral().
+
+    \note In case Bluetooth backend you are using does not support reading RSSI,
+    the errorOccurred() signal is emitted with an error code QLowEnergyController::RssiReadError.
+    At the moment platforms supporting reading RSSI include Android, iOS and macOS.
+
+    \sa rssiRead(), connectToDevice(), state(), createCentral(), errorOccurred()
+    \since 6.5
+*/
+void QLowEnergyController::readRssi()
+{
+    if (role() != CentralRole) {
+        qCWarning(QT_BT, "Invalid role (peripheral), cannot read RSSI");
+        return d_ptr->setError(RssiReadError); // This also emits.
+    }
+
+    switch (state()) {
+    case UnconnectedState:
+    case ConnectingState:
+    case ClosingState:
+        qCWarning(QT_BT, "Cannot read RSSI while not in 'Connected' state, connect first");
+        return d_ptr->setError(RssiReadError); // Will emit.
+    default:
+        d_ptr->readRssi();
+    }
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qlowenergycontroller.cpp"
