@@ -18,6 +18,8 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_BLUEZ)
 
+using namespace QtBluetoothPrivate; // for D-Bus wrappers
+
 QLowEnergyControllerPrivateBluezDBus::QLowEnergyControllerPrivateBluezDBus(
         const QString &adapterPathWithPeripheralSupport)
     : QLowEnergyControllerPrivate(),
@@ -644,7 +646,6 @@ void QLowEnergyControllerPrivateBluezDBus::discoverServiceDetails(
         return;
     }
 
-    QStringList descriptorPaths;
     const ManagedObjectList managedObjectList = reply.value();
     for (ManagedObjectList::const_iterator it = managedObjectList.constBegin(); it != managedObjectList.constEnd(); ++it) {
         const InterfaceList &ifaceList = it.value();
