@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
 
@@ -82,7 +82,7 @@ void tst_QBluetoothLocalDevice::tst_powerOn()
 {
     if (androidBluetoothEmulator())
         QSKIP("Skipping test on Android 12+ emulator, CI can timeout waiting for user input");
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     QSKIP("Not possible on OS X");
 #endif
     if (numDevices == 0)
@@ -116,7 +116,7 @@ void tst_QBluetoothLocalDevice::tst_powerOff()
 {
     if (androidBluetoothEmulator())
         QSKIP("Skipping test on Android 12+ emulator, CI can timeout waiting for user input");
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     QSKIP("Not possible on OS X");
 #endif
     if (numDevices == 0)
@@ -192,7 +192,7 @@ void tst_QBluetoothLocalDevice::tst_hostModes()
 {
     if (androidBluetoothEmulator())
         QSKIP("Skipping test on Android 12+ emulator, CI can timeout waiting for user input");
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     QSKIP("Not possible on OS X");
 #endif
     QFETCH(QBluetoothLocalDevice::HostMode, hostModeExpected);
